@@ -7,13 +7,16 @@ use std::env;
 fn main() {
     let target = env::var("TARGET").unwrap();
     let aarch64 = target.contains("aarch64");
+    let i686 = target.contains("i686");
     let x86_64 = target.contains("x86_64");
     let windows = target.contains("windows");
     let mingw = target.contains("windows-gnu");
     let linux = target.contains("unknown-linux");
     let android = target.contains("android");
     let apple = target.contains("apple");
-    let musl = target.contains("musl");
+    let emscripten = target.contains("asm");
+    let musl = target.contains("musl") || emscripten;
+    let uclibc = target.contains("uclibc");
     let freebsd = target.contains("freebsd");
     let dragonfly = target.contains("dragonfly");
     let mips = target.contains("mips");
@@ -24,7 +27,7 @@ fn main() {
     let mut cfg = ctest::TestGenerator::new();
 
     // Pull in extra goodies
-    if linux || android {
+    if linux || android || emscripten {
         cfg.define("_GNU_SOURCE", None);
     } else if netbsd {
         cfg.define("_NETBSD_SOURCE", Some("1"));
@@ -77,6 +80,7 @@ fn main() {
         cfg.header("netinet/in.h");
         cfg.header("netinet/ip.h");
         cfg.header("netinet/tcp.h");
+        cfg.header("netinet/udp.h");
         cfg.header("resolv.h");
         cfg.header("pthread.h");
         cfg.header("dlfcn.h");
@@ -87,6 +91,9 @@ fn main() {
         cfg.header("sys/mman.h");
         cfg.header("sys/resource.h");
         cfg.header("sys/socket.h");
+        if linux && !musl {
+            cfg.header("linux/if.h");
+        }
         cfg.header("sys/time.h");
         cfg.header("sys/un.h");
         cfg.header("sys/wait.h");
@@ -104,6 +111,7 @@ fn main() {
         cfg.header("syslog.h");
         cfg.header("semaphore.h");
         cfg.header("sys/statvfs.h");
+        cfg.header("sys/times.h");
     }
 
     if android {
@@ -115,6 +123,9 @@ fn main() {
         cfg.header("arpa/inet.h");
         cfg.header("xlocale.h");
         cfg.header("utmp.h");
+        if i686 || x86_64 {
+            cfg.header("sys/reg.h");
+        }
     } else if !windows {
         cfg.header("glob.h");
         cfg.header("ifaddrs.h");
@@ -126,8 +137,10 @@ fn main() {
 
         if !musl {
             cfg.header("sys/sysctl.h");
+        }
+        if !musl && !uclibc {
 
-            if !netbsd && !openbsd {
+            if !netbsd && !openbsd && !uclibc {
                 cfg.header("execinfo.h");
                 cfg.header("xlocale.h");
             }
@@ -145,9 +158,14 @@ fn main() {
         cfg.header("mach/mach_time.h");
         cfg.header("malloc/malloc.h");
         cfg.header("util.h");
+        cfg.header("sys/xattr.h");
+        cfg.header("sys/sys_domain.h");
         if target.starts_with("x86") {
             cfg.header("crt_externs.h");
         }
+        cfg.header("net/route.h");
+        cfg.header("net/route.h");
+        cfg.header("sys/proc_info.h");
     }
 
     if bsdlike {
@@ -160,20 +178,34 @@ fn main() {
         }
     }
 
-    if linux {
+    if linux || emscripten {
         cfg.header("mqueue.h");
         cfg.header("ucontext.h");
-        cfg.header("sys/signalfd.h");
-        cfg.header("sys/xattr.h");
+        if !uclibc {
+            // optionally included in uclibc
+            cfg.header("sys/xattr.h");
+        }
         cfg.header("sys/ipc.h");
+        cfg.header("sys/sem.h");
         cfg.header("sys/msg.h");
         cfg.header("sys/shm.h");
+        cfg.header("sys/user.h");
         cfg.header("sys/fsuid.h");
-        cfg.header("pty.h");
+        cfg.header("sys/timerfd.h");
         cfg.header("shadow.h");
+        if !emscripten {
+            cfg.header("linux/input.h");
+            cfg.header("linux/falloc.h");
+        }
+        if x86_64 {
+            cfg.header("sys/io.h");
+        }
+        if i686 || x86_64 {
+            cfg.header("sys/reg.h");
+        }
     }
 
-    if linux || android {
+    if linux || android || emscripten {
         cfg.header("malloc.h");
         cfg.header("net/ethernet.h");
         cfg.header("netpacket/packet.h");
@@ -182,11 +214,21 @@ fn main() {
         cfg.header("sys/eventfd.h");
         cfg.header("sys/prctl.h");
         cfg.header("sys/sendfile.h");
+        cfg.header("sys/signalfd.h");
         cfg.header("sys/vfs.h");
         cfg.header("sys/syscall.h");
-        cfg.header("sys/sysinfo.h");
+        cfg.header("sys/personality.h");
+        cfg.header("sys/swap.h");
+        cfg.header("pty.h");
+        if !uclibc {
+            cfg.header("sys/sysinfo.h");
+        }
         cfg.header("sys/reboot.h");
+        if !emscripten {
+            cfg.header("linux/netfilter_ipv4.h");
+        }
         if !musl {
+            cfg.header("asm/mman.h");
             cfg.header("linux/netlink.h");
             cfg.header("linux/magic.h");
             cfg.header("linux/reboot.h");
@@ -197,6 +239,13 @@ fn main() {
         }
     }
 
+    if linux || android {
+        // DCCP support
+        if !uclibc && !musl && !emscripten {
+            cfg.header("linux/dccp.h");
+        }
+    }
+
     if freebsd {
         cfg.header("pthread_np.h");
         cfg.header("sched.h");
@@ -211,6 +260,9 @@ fn main() {
         cfg.header("ufs/ufs/quota.h");
         cfg.header("ufs/ufs/quota1.h");
         cfg.header("sys/ioctl_compat.h");
+
+        // DCCP support
+        cfg.header("netinet/dccp.h");
     }
 
     if openbsd {
@@ -225,8 +277,10 @@ fn main() {
         cfg.header("sys/ioctl_compat.h");
     }
 
-    if linux || freebsd || dragonfly || netbsd || apple {
-        cfg.header("aio.h");
+    if linux || freebsd || dragonfly || netbsd || apple || emscripten {
+        if !uclibc {
+            cfg.header("aio.h");
+        }
     }
 
     cfg.type_name(move |ty, is_struct| {
@@ -282,6 +336,9 @@ fn main() {
                 }
             }
             "u64" if struct_ == "epoll_event" => "data.u64".to_string(),
+            "type_" if linux &&
+                (struct_ == "input_event" || struct_ == "input_mask" ||
+                 struct_ == "ff_effect") => "type".to_string(),
             s => s.to_string(),
         }
     });
@@ -310,6 +367,15 @@ fn main() {
             // This is actually a union, not a struct
             "sigval" => true,
 
+            // Linux kernel headers used on musl are too old to have this
+            // definition. Because it's tested on other Linux targets, skip it.
+            "input_mask" if musl => true,
+
+            // These structs have changed since unified headers in NDK r14b.
+            // `st_atime` and `st_atime_nsec` have changed sign.
+            // FIXME: unskip it for next major release
+            "stat" | "stat64" if android => true,
+
             _ => false
         }
     });
@@ -364,12 +430,6 @@ fn main() {
             "NOTE_EXIT_REPARENTED" |
             "NOTE_REAP" if apple => true,
 
-            // The linux/quota.h header file which defines these can't be
-            // included with sys/quota.h currently on MIPS, so we don't include
-            // it and just ignore these constants
-            "QFMT_VFS_OLD" |
-            "QFMT_VFS_V0" if mips && linux => true,
-
             // These constants were removed in FreeBSD 11 (svn r273250) but will
             // still be accepted and ignored at runtime.
             "MAP_RENAME" |
@@ -388,6 +448,45 @@ fn main() {
             "KERN_KDENABLE_BG_TRACE" if apple => true,
             "KERN_KDDISABLE_BG_TRACE" if apple => true,
 
+            // These constants were removed in OpenBSD 6 (https://git.io/v7gBO
+            // https://git.io/v7gBq)
+            "KERN_USERMOUNT" |
+            "KERN_ARND" if openbsd => true,
+
+            // These constats were added in OpenBSD 6.2
+            "EV_RECEIPT" | "EV_DISPATCH" if openbsd => true,
+
+            // These are either unimplemented or optionally built into uClibc
+            "LC_CTYPE_MASK" | "LC_NUMERIC_MASK" | "LC_TIME_MASK" | "LC_COLLATE_MASK" | "LC_MONETARY_MASK" | "LC_MESSAGES_MASK" |
+            "MADV_MERGEABLE" | "MADV_UNMERGEABLE" | "MADV_HWPOISON" | "IPV6_ADD_MEMBERSHIP" | "IPV6_DROP_MEMBERSHIP" | "IPV6_MULTICAST_LOOP" | "IPV6_V6ONLY" |
+            "MAP_STACK" | "RTLD_DEEPBIND" | "SOL_IPV6" | "SOL_ICMPV6" if uclibc => true,
+
+            // Musl uses old, patched kernel headers
+            "FALLOC_FL_COLLAPSE_RANGE" | "FALLOC_FL_ZERO_RANGE" |
+            "FALLOC_FL_INSERT_RANGE" | "FALLOC_FL_UNSHARE_RANGE" if musl => true,
+
+            // Defined by libattr not libc on linux (hard to test).
+            // See constant definition for more details.
+            "ENOATTR" if linux => true,
+
+            // On mips*-unknown-linux-gnu* CMSPAR cannot be included with the set of headers we
+            // want to use here for testing. It's originally defined in asm/termbits.h, which is
+            // also included by asm/termios.h, but not the standard termios.h. There's no way to
+            // include both asm/termbits.h and termios.h and there's no way to include both
+            // asm/termios.h and ioctl.h (+ some other headers) because of redeclared types.
+            "CMSPAR" if mips && linux && !musl => true,
+
+            // On mips Linux targets, MADV_SOFT_OFFLINE is currently missing, though it's been added but CI has too old
+            // of a Linux version. Since it exists on all other Linux targets, just ignore this for now and remove once
+            // it's been fixed in CI.
+            "MADV_SOFT_OFFLINE" if mips && linux => true,
+
+            // These constants are tested in a separate test program generated below because there
+            // are header conflicts if we try to include the headers that define them here.
+            "F_CANCELLK" | "F_ADD_SEALS" | "F_GET_SEALS" => true,
+            "F_SEAL_SEAL" | "F_SEAL_SHRINK" | "F_SEAL_GROW" | "F_SEAL_WRITE" => true,
+            "QFMT_VFS_OLD" | "QFMT_VFS_V0" | "QFMT_VFS_V1" if mips && linux => true, // Only on MIPS
+
             _ => false,
         }
     });
@@ -398,7 +497,8 @@ fn main() {
             "execv" |       // crazy stuff with const/mut
             "execve" |
             "execvp" |
-            "execvpe" => true,
+            "execvpe" |
+            "fexecve" => true,
 
             "getrlimit" | "getrlimit64" |    // non-int in 1st arg
             "setrlimit" | "setrlimit64" |    // non-int in 1st arg
@@ -474,6 +574,17 @@ fn main() {
             // it's in a header file?
             "endpwent" if android => true,
 
+
+            // These are either unimplemented or optionally built into uClibc
+            // or "sysinfo", where it's defined but the structs in linux/sysinfo.h and sys/sysinfo.h
+            // clash so it can't be tested
+            "getxattr" | "lgetxattr" | "fgetxattr" | "setxattr" | "lsetxattr" | "fsetxattr" |
+            "listxattr" | "llistxattr" | "flistxattr" | "removexattr" | "lremovexattr" |
+            "fremovexattr" |
+            "backtrace" |
+            "sysinfo" | "newlocale" | "duplocale" | "freelocale" | "uselocale" |
+            "nl_langinfo_l" | "wcslen" | "wcstombs" if uclibc => true,
+
             // Apparently res_init exists on Android, but isn't defined in a header:
             // https://mail.gnome.org/archives/commits-list/2013-May/msg01329.html
             "res_init" if android => true,
@@ -484,6 +595,19 @@ fn main() {
             // https://github.com/rust-lang/libc/pull/585#discussion_r114561460
             "res_init" if apple => true,
 
+            // On Mac we don't use the default `close()`, instead using their $NOCANCEL variants.
+            "close" if apple => true,
+
+            // Definition of those functions as changed since unified headers from NDK r14b
+            // These changes imply some API breaking changes but are still ABI compatible.
+            // We can wait for the next major release to be compliant with the new API.
+            // FIXME: unskip these for next major release
+            "strerror_r" | "madvise" | "msync" | "mprotect" | "recvfrom" | "getpriority" |
+            "setpriority" | "personality" if android => true,
+            // In Android 64 bits, these functions have been fixed since unified headers.
+            // Ignore these until next major version.
+            "bind" | "writev" | "readv" | "sendmsg" | "recvmsg" if android && (aarch64 || x86_64) => true,
+
             _ => false,
         }
     });
@@ -509,7 +633,11 @@ fn main() {
         // aio_buf is "volatile void*" and Rust doesn't understand volatile
         (struct_ == "aiocb" && field == "aio_buf") ||
         // stack_t.ss_sp's type changed from FreeBSD 10 to 11 in svn r294930
-        (freebsd && struct_ == "stack_t" && field == "ss_sp")
+        (freebsd && struct_ == "stack_t" && field == "ss_sp") ||
+        // type siginfo_t.si_addr changed from OpenBSD 6.0 to 6.1
+        (openbsd && struct_ == "siginfo_t" && field == "si_addr") ||
+        // this one is an anonymous union
+        (linux && struct_ == "ff_effect" && field == "u")
     });
 
     cfg.skip_field(move |struct_, field| {
@@ -532,9 +660,37 @@ fn main() {
         }
     });
 
-    if env::var("SKIP_COMPILE").is_ok() {
-        cfg.generate_files("../src/lib.rs", "all.rs");
+    cfg.generate("../src/lib.rs", "main.rs");
+
+    // On Linux or Android also generate another script for testing linux/fcntl declarations.
+    // These cannot be tested normally because including both `linux/fcntl.h` and `fcntl.h`
+    // fails on a lot of platforms.
+    let mut cfg = ctest::TestGenerator::new();
+    cfg.skip_type(|_| true)
+        .skip_struct(|_| true)
+        .skip_fn(|_| true);
+    if android || linux {
+        // musl defines these directly in `fcntl.h`
+        if musl {
+            cfg.header("fcntl.h");
+        } else {
+            cfg.header("linux/fcntl.h");
+        }
+        if !musl {
+            cfg.header("net/if.h");
+            cfg.header("linux/if.h");
+        }
+        cfg.header("linux/quota.h");
+        cfg.skip_const(move |name| {
+            match name {
+                "F_CANCELLK" | "F_ADD_SEALS" | "F_GET_SEALS" => false,
+                "F_SEAL_SEAL" | "F_SEAL_SHRINK" | "F_SEAL_GROW" | "F_SEAL_WRITE" => false,
+                "QFMT_VFS_OLD" | "QFMT_VFS_V0" | "QFMT_VFS_V1" if mips && linux => false,
+                _ => true,
+            }
+        });
     } else {
-        cfg.generate("../src/lib.rs", "all.rs");
+        cfg.skip_const(|_| true);
     }
+    cfg.generate("../src/lib.rs", "linux_fcntl.rs");
 }
